@echo off
setlocal

echo **********************************************
echo  BOUSI-PROLOG source v4.0 installation script
echo **********************************************

rem Gets the full path where this batch file is located
set SCRIPTDIR=%~dp0
set SCRIPTDIR=%SCRIPTDIR:~,-1%

rem Default installation directory is "C:\Bousi-Prolog"
set INSTALLDIR=c:\Bousi-Prolog


:ChooseInstallPath

rem Asks for the installation directory
echo.
echo Bousi-Prolog source will be installed at %INSTALLDIR%
echo Make sure you have write access to the selected directory.

:InstallPathLoop
set OPTION=
set /p OPTION="Do you want to use this installation path ([Y]/N)? "
if "%OPTION%"=="N" goto ChangeInstallPath
if "%OPTION%"=="n" goto ChangeInstallPath
if "%OPTION%"=="Y" goto AcceptInstallPath
if "%OPTION%"=="y" goto AcceptInstallPath
if "%OPTION%"==""  goto AcceptInstallPath
goto InstallPathLoop

:ChangeInstallPath
rem Changes the installation path
set /p INSTALLDIR="Enter the installation path: "
goto ChooseInstallPath


:AcceptInstallPath

rem Copies source code files
echo.
echo Copying source code files...
xcopy "%SCRIPTDIR%\files" "%INSTALLDIR%" /y /s /i
if errorlevel 1 goto FileCopyError

rem Asks whether the installer should create shorcuts for this user,
rem all users or nobody
echo.

:ShortcutsLoop
set OPTION=
set /p OPTION="Do you want to create shortcuts for this user, all users or none ([T]/A/N)? "
if "%OPTION%"=="A" goto AllUsersShortcuts
if "%OPTION%"=="a" goto AllUsersShortcuts
if "%OPTION%"=="T" goto ThisUserShortcuts
if "%OPTION%"=="t" goto ThisUserShortcuts
if "%OPTION%"=="N" goto SkipShortcuts
if "%OPTION%"=="n" goto SkipShortcuts
if "%OPTION%"==""  goto ThisUserShortcuts
goto ShortcutsLoop

:AllUsersShortcuts
rem The shortcuts will be added to the "All Users\Start Menu\Programs" folder
echo.
echo Creating shortcuts for all users in the Start Menu...
for /F "tokens=* delims= " %%A in ('cscript /Nologo "%SCRIPTDIR%\scripts\folder.vbs" AllUsersPrograms') do (
	set ALLUSERSPROGRAMSFOLDER=%%A
)
set SHORTCUTPATH=%ALLUSERSPROGRAMSFOLDER%\Bousi-Prolog
goto CreateShortcuts

:ThisUserShortcuts
rem The shortcuts will be added to the "<user>\Start Menu\Programs" folder
echo.
echo Creating shortcuts for the current user in the Start Menu...
for /F "tokens=* delims= " %%A in ('cscript /Nologo "%SCRIPTDIR%\scripts\folder.vbs" Programs') do (
	set PROGRAMSFOLDER=%%A
)
set SHORTCUTPATH=%PROGRAMSFOLDER%\Bousi-Prolog
goto CreateShortcuts

:CreateShortcuts
rem Creates the shortcuts
set SHORTCUTS=1
wscript "%SCRIPTDIR%\scripts\shortcut.vbs" "%SHORTCUTPATH%\Run Bousi-Prolog.lnk" "%INSTALLDIR%\bousi.exe" "%INSTALLDIR%" "" "Run the Bousi-Prolog system" "cmd.exe" "1"
wscript "%SCRIPTDIR%\scripts\shortcut.vbs" "%SHORTCUTPATH%\Build Bousi-Prolog.lnk" "%INSTALLDIR%\make.bat" "%INSTALLDIR%" "" "Build the Bousi-Prolog system" "" "1"
wscript "%SCRIPTDIR%\scripts\shortcut.vbs" "%SHORTCUTPATH%\Open Bousi-Prolog directory.lnk" "explorer.exe" "%INSTALLDIR%" "%INSTALLDIR%" "Open the Bousi-Prolog installation directory" "explorer.exe,13" "1"
wscript "%SCRIPTDIR%\scripts\shortcut.vbs" "%SHORTCUTPATH%\Uninstall Bousi-Prolog.lnk" "%INSTALLDIR%\uninstaller.bat" "%INSTALLDIR%" "" "Uninstall the Bousi-Prolog system" "" "1"
goto BuildBPL

:SkipShortcuts
rem Doesn't create any shortcut
set SHORTCUTS=0
goto BuildBPL

:BuildBPL

rem Asks whether the source code should be compiled now
echo.

:CompileLoop
set OPTION=
set /p OPTION="Do you want to compile Bousi-Prolog now ([Y]/N)? "
if "%OPTION%"==""  goto CompileNow
if "%OPTION%"=="Y" goto CompileNow
if "%OPTION%"=="y" goto CompileNow
if "%OPTION%"=="N" goto SkipCompilation
if "%OPTION%"=="n" goto SkipCompilation
goto CompileLoop

:CompileNow

rem Compiles the Bousi-Prolog system
echo.
echo Building Bousi-Prolog system...

cd /d "%INSTALLDIR%"
call make.bat all
cd /d "%SCRIPTDIR%"
set COMPILED=1
goto Instructions

:SkipCompilation
rem Skips the compilation
set COMPILED=0
goto Instructions

:Instructions

rem Shows how to compile and/or start Bousi-Prolog
echo.
if "%COMPILED%%SHORTCUTS%"=="00" (
	echo Bousi-Prolog source code has been installed.
	echo Go to the installation directory and run the "make" batch file to compile
	echo the source code. Then, you will be able to launch the Bousi-Prolog system
	echo running the "bousi" executable file.
) else if "%COMPILED%%SHORTCUTS%"=="01" (
	echo Bousi-Prolog source code has been installed.
	echo Run the "Build Bousi-Prolog" shortcut to compile the source code. Then,
	echo you will be able to launch the Bousi-Prolog system using the "Run Bousi-
	echo Prolog" shortcut.
) else if "%COMPILED%%SHORTCUTS%"=="10" (
	echo Bousi-Prolog source code has been installed and compiled.
	echo Go to the installation directory and run the "bousi" executable file to
	echo launch the Bousi-Prolog system.
) else if "%COMPILED%%SHORTCUTS%"=="11" (
	echo Bousi-Prolog source code has been installed and compiled.
	echo Use the "Run Bousi-Prolog" shortcut to launch the Bousi-Prolog system.
)
rem Sets the installation directory as the new working directory
cd /d "%INSTALLDIR%"
goto End

:FileCopyError

rem There was an error during file copy
echo.
echo Installation aborted: File copy error

:End

echo.
echo Press any key to exit the installation script.
pause > nul

